﻿using Dynamicweb.Core;
using Dynamicweb.Ecommerce.LiveIntegration.Logging;
using System;
using System.IO;

namespace Dynamicweb.Ecommerce.LiveIntegration.Configuration
{
    /// <summary>
    /// Configuration file handler.
    /// </summary>
    internal static class SettingsHandler
    {
        /// <summary>
        /// Gets the virtual configuration file path .
        /// </summary>
        private static readonly string ConfigFilePathVirtual = $"/Files/System/LiveIntegration/{Constants.AddInName}.Setup.xml";

        /// <summary>
        /// Gets the physical configuration file path .
        /// </summary>
        private static string _configFilePathPhysical;

        /// <summary>
        /// A file system watcher to watch for changes to the settings file.
        /// </summary>
        private static FileSystemWatcher _fileSystemWatcher;

        /// <summary>
        /// Initializes static members of the <see cref="SettingsHandler"/> class.
        /// </summary>
        static SettingsHandler()
        {
            EnsureConfigurationFolderExists();
            SetUpWatching();
        }

        /// <summary>
        /// Gets the physical path of the configuration file.
        /// </summary>
        /// <value>The configuration file path physical.</value>
        private static string ConfigFilePathPhysical
        {
            get
            {
                if (_configFilePathPhysical == null)
                {
                    _configFilePathPhysical = SystemInformation.MapPath(ConfigFilePathVirtual);
                }

                return _configFilePathPhysical;
            }
        }

        /// <summary>
        /// Loads the settings from disk.
        /// </summary>
        /// <exception cref="Exception"></exception>
        public static Settings LoadSettings()
        {
            Settings settings = null;
            if (_fileSystemWatcher != null)
                _fileSystemWatcher.EnableRaisingEvents = false;

            try
            {
                if (!File.Exists(ConfigFilePathPhysical))
                {
                    settings = new Settings();
                }
                else
                {
                    var xml = File.ReadAllText(ConfigFilePathPhysical);
                    var serializer = new SettingsSerializer();
                    settings = serializer.Deserialize(xml);
                }
            }
            catch
            {
                string errorMessage = $"Error reading live integration file at {ConfigFilePathPhysical}. Make sure the file exists, is accessible and is currently not in use.";
                throw new Exception(errorMessage);
            }
            finally
            {
                if (_fileSystemWatcher != null)
                    _fileSystemWatcher.EnableRaisingEvents = true;
            }

            return settings;
        }

        /// <summary>
        /// Saves the settings to disk.
        /// </summary>
        /// <param name="settings">The settings that need to be saved.</param>
        public static void SaveSettings(Settings settings)
        {
            var serializer = new SettingsSerializer();
            var xml = serializer.Serialize(settings);

            try
            {
                if (_fileSystemWatcher != null)
                    _fileSystemWatcher.EnableRaisingEvents = false;

                File.WriteAllText(ConfigFilePathPhysical, xml);
            }
            catch
            {
                string errorMessage =
                    $"Error writing live integration file at {ConfigFilePathPhysical}. Make sure the file exists, is accessible and is currently not in use.";
                Logger.Instance.Log(ErrorLevel.Error, errorMessage);
                throw new Exception(errorMessage);
            }
            finally
            {
                Settings.Reload();
                if (_fileSystemWatcher != null)
                    _fileSystemWatcher.EnableRaisingEvents = true;
            }
        }

        /// <summary>
        /// Ensures the configuration folder exists.
        /// </summary>
        internal static void EnsureConfigurationFolderExists()
        {
            var folderPath = Path.GetDirectoryName(ConfigFilePathPhysical);
            if (!string.IsNullOrEmpty(folderPath))
            {
                Directory.CreateDirectory(folderPath);
            }
        }

        /// <summary>
        /// Handles file change events and reload the settings.
        /// </summary>
        /// <param name="source">The source.</param>
        /// <param name="e">The <see cref="FileSystemEventArgs"/> instance containing the event data.</param>
        private static void FileSystemWatcherOnChanged(object source, FileSystemEventArgs e)
        {
            if (_fileSystemWatcher != null)
                _fileSystemWatcher.EnableRaisingEvents = false;
            try
            {
                Settings.Reload();
                System.Threading.Thread.Sleep(100);
                Logger.Instance.Log(ErrorLevel.DebugInfo, $"Settings reset because of file watcher event: {e.ChangeType}.");
            }
            finally
            {
                if (_fileSystemWatcher != null)
                {
                    if (e.ChangeType != WatcherChangeTypes.Deleted)
                    {
                        _fileSystemWatcher.EnableRaisingEvents = true;
                    }
                    else
                    {
                        _fileSystemWatcher.Dispose();
                        _fileSystemWatcher = null;
                    }
                }
            }
        }

        /// <summary>
        /// Handles exceptions raised by the file system watcher.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="errorEventArgs">The <see cref="ErrorEventArgs"/> instance containing the event data.</param>
        private static void FileSystemWatcherOnError(object sender, ErrorEventArgs errorEventArgs)
        {
            string errorMessage = $"Error watching Live Integration settings XML file. Error: {errorEventArgs.GetException()}";
            Logger.Instance.Log(ErrorLevel.Error, errorMessage);
            SetUpWatching();
        }

        /// <summary>
        /// Configures file watching.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Reliability", "CA2000:Dispose objects before losing scope")]
        private static void SetUpWatching()
        {
            _fileSystemWatcher = new FileSystemWatcher
            {
                NotifyFilter = NotifyFilters.LastWrite | NotifyFilters.FileName,
                Path = Path.GetDirectoryName(ConfigFilePathPhysical),
                Filter = Path.GetFileName(ConfigFilePathPhysical)
            };

            // Add event handlers.
            _fileSystemWatcher.Changed += FileSystemWatcherOnChanged;
            _fileSystemWatcher.Deleted += FileSystemWatcherOnChanged;
            _fileSystemWatcher.Renamed += FileSystemWatcherOnChanged;
            _fileSystemWatcher.Error += FileSystemWatcherOnError;

            // Begin watching.
            _fileSystemWatcher.EnableRaisingEvents = true;
        }
    }
}