﻿using Dynamicweb.Ecommerce.LiveIntegration.Configuration;
using Dynamicweb.Ecommerce.LiveIntegration.EndpointMonitoring;
using Dynamicweb.Ecommerce.LiveIntegration.Licensing;
using Dynamicweb.Ecommerce.LiveIntegration.Logging;
using Dynamicweb.Extensibility.Notifications;
using System;
using System.Collections.Generic;
using System.Linq;
using static Dynamicweb.Ecommerce.LiveIntegration.Notifications.Communication;
namespace Dynamicweb.Ecommerce.LiveIntegration.Connectors
{
    /// <summary>
    /// Abstract base connector
    /// </summary>
    internal abstract class ConnectorBase
    {
        internal abstract EndpointInfo GetEndpoint();
        internal abstract string Execute(string request);
        internal abstract string Execute(EndpointInfo endpoint, string request, TimeSpan responseTimeout);

        internal abstract bool IsConnectionAvailableFromBackend(string endpointId, out string error);

        /// <summary>
        /// Determines whether the web service connection is available.
        /// </summary>
        internal abstract bool IsWebServiceConnectionAvailable(EndpointInfo endpoint);

        internal abstract IEnumerable<string> GetUrls(string multipleUrlsText);

        /// <summary>
        /// Clears the cache.
        /// </summary>
        internal void ClearCache()
        {
            EndpointMonitoringService.ClearEndpoints();
        }

        internal void Error(EndpointInfo endpoint)
        {
            EndpointMonitoringService.Error(endpoint);
        }

        internal string Execute(EndpointInfo endpoint, string request)
        {
            return Execute(endpoint, request, TimeSpan.Zero);
        }

        protected bool ExecuteConnectionAvailableRequest(EndpointInfo endpoint, out Exception error)
        {
            error = null;
            bool connectionAvailable = false;
            if (EndpointMonitoringService.IsStillValid(endpoint, out connectionAvailable))
            {
                return connectionAvailable;
            }
            Diagnostics.ExecutionTable.Current.Add("LiveIntegration.ConnectorBase.IsWebServiceConnectionAvailable(endpoint) START");
            var lastStatus = EndpointMonitoringService.GetEndpointStatus(endpoint);
            bool isWebServiceConnectionAvailable = (lastStatus != null) ? !lastStatus.Error : false;
            DateTime? lastErpCommunication = (lastStatus != null) ? lastStatus.LastErpCommunication : null;
            bool success = false;

            string request = "<GetEcomData></GetEcomData>"; // simple request for checking connection
            string response = null;
            try
            {
                TimeSpan ts = TimeSpan.Zero;
                if (Settings.Instance.ConnectionTimeout > 0)
                {
                    ts = new TimeSpan(0, 0, Settings.Instance.ConnectionTimeout);
                }
                response = Execute(endpoint, request, ts);
                success = true;
                EndpointMonitoringService.Success(endpoint);
            }
            catch (Exception ex)
            {
                error = ex;
                Logger.Instance.Log(ErrorLevel.ConnectionError, $"Error checking Web Service connection: '{ex.Message}'. Request: '{request}'.");

                EndpointMonitoringService.Error(endpoint);

                NotificationManager.Notify(OnErpCommunicationLost, new OnErpCommunicationLostArgs(request, ex));

                if (Connector.EnableThrowExceptions)
                {
                    throw new Exception("Thrown exception", ex);
                }
            }
            if (success && lastErpCommunication.HasValue && !isWebServiceConnectionAvailable)
            {
                NotificationManager.Notify(OnErpCommunicationRestored, new OnErpCommunicationRestoredArgs(request, lastErpCommunication));
            }
            if (success && !string.IsNullOrEmpty(response))
            {
                LicenseService.ValidateLicense(endpoint.GetUrl(), response);
            }
            Diagnostics.ExecutionTable.Current.Add("LiveIntegration.ConnectorBase.IsWebServiceConnectionAvailable(endpoint) END");
            return success;
        }

        internal bool IsConnectionAvailableFromBackend(string multipleUrlsText)
        {
            bool result = false;
            List<string> errorsList = new List<string>();
            IEnumerable<string> urls = null;
            if (!string.IsNullOrEmpty(multipleUrlsText))
            {
                result = true;
                urls = GetUrls(multipleUrlsText);
                foreach (string endpointId in urls)
                {
                    string message = null;
                    bool isAvailable = IsConnectionAvailableFromBackend(endpointId, out message);
                    if (isAvailable)
                    {
                        result = result == false ? false : true;
                    }
                    else
                    {
                        errorsList.Add(message);
                        result = false;
                    }
                }
            }
            if (!result && urls?.Count() > 1 && errorsList.Count > 0)
            {
                throw new Exception(string.Join(System.Environment.NewLine, errorsList));
            }
            return result;
        }
    }
}
