﻿
using Dynamicweb.DataIntegration.EndpointManagement;
using Dynamicweb.Ecommerce.LiveIntegration.Configuration;
using Dynamicweb.Ecommerce.LiveIntegration.Connectors;
using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace Dynamicweb.Ecommerce.LiveIntegration.EndpointMonitoring
{
    /// <summary>
    /// EndpointMonitoringService
    /// </summary>
    internal class EndpointMonitoringService
    {
        private static ConcurrentDictionary<string, EndpointStatus> EndpointCollection = new ConcurrentDictionary<string, EndpointStatus>();
        private static readonly List<EndpointInfo> EndpointsInPing = new List<EndpointInfo>();

        private static readonly object SyncRoot = new object();

        /// <summary>
        /// Checks if endpoint connection status is still valid
        /// </summary>        
        /// <param name="endpoint">The endpoint</param>
        /// <param name="connectionAvailable">connection available result</param>
        /// <returns>True if status is still valid otherwise false</returns>
        public static bool IsStillValid(EndpointInfo endpoint, out bool connectionAvailable)
        {
            connectionAvailable = false;

            if (EndpointsInPing.Contains(endpoint))
            {
                return true;
            }

            EndpointStatus endpointStatus = null;
            if (EndpointCollection.TryGetValue(endpoint.GetUrl(), out endpointStatus))
            {
                connectionAvailable = !endpointStatus.Error;
                return endpointStatus.IsStillValid();
            }
            else
            {
                return false;
            }
        }

        /// <summary>
        /// Returns current endpoint status
        /// </summary>        
        /// <param name="endpoint">The endpoint</param>
        /// <returns>Endpoint connection status if it exists otherwise null</returns>
        public static EndpointStatus GetEndpointStatus(EndpointInfo endpoint)
        {
            EndpointStatus status = null;
            if (EndpointCollection.TryGetValue(endpoint.GetUrl(), out status))
            {
                return status;
            }
            return null;
        }

        /// <summary>
        /// Sets endpoint communication to success.
        /// </summary>        
        /// <param name="endpoint">The endpoint</param>
        public static void Success(EndpointInfo endpoint)
        {
            EndpointStatus status = null;
            string url = endpoint.GetUrl();
            if (!EndpointCollection.TryGetValue(url, out status))
            {
                status = new EndpointStatus();
                EndpointCollection.TryAdd(url, status);
            }
            status.SetSuccess();
        }

        /// <summary>
        /// Sets endpoint communication failed.
        /// </summary>        
        /// <param name="endpoint">The endpoint</param>
        public static void Error(EndpointInfo endpoint)
        {
            EndpointStatus status = null;
            string url = endpoint.GetUrl();
            if (!EndpointCollection.TryGetValue(url, out status))
            {
                status = new EndpointStatus();
                EndpointCollection.TryAdd(url, status);
            }
            if (!EndpointsInPing.Contains(endpoint) && !Environment.ExecutingContext.IsBackEnd())
            {
                Task.Run(() => PingEndpoint(EndpointCollection, endpoint));
            }
            status.SetError();
        }

        /// <summary>
        /// Clears endpoints statuses collection
        /// </summary>
        public static void ClearEndpoints()
        {
            EndpointCollection.Clear();
        }

        /// <summary>
        /// Pings endpoind during ping interval
        /// </summary>
        /// <param name="endpointCollection"></param> 
        /// <param name="endpoint"></param>
        private static void PingEndpoint(ConcurrentDictionary<string, EndpointStatus> endpointCollection, EndpointInfo endpoint)
        {
            var autoPingInterval = Settings.Instance.AutoPingInterval;
            if (autoPingInterval <= 0)
            {
                return;
            }
            if (!EndpointsInPing.Contains(endpoint))
            {
                lock (SyncRoot)
                {
                    EndpointStatus status = null;
                    if (!endpointCollection.TryGetValue(endpoint.GetUrl(), out status) || (status != null && status.Error))
                    {
                        EndpointsInPing.Add(endpoint);
                        System.Threading.Thread.Sleep(autoPingInterval * 1000);
                        EndpointsInPing.Remove(endpoint);
                        Connector.IsWebServiceConnectionAvailable(endpoint);
                    }
                }
            }
        }
    }
}
